/*

    Zelda Navi's Quest

    Copyright (C) 2013-2014  Vincent Jouillat

    Please send bugreports with examples or suggestions to www.zeldaroth.fr

*/

#ifndef __COMMONGAME_H__
#define __COMMONGAME_H__

#define MAX_CASE_X          400
#define MAX_CASE_Y          400

#define ANIM_REFRESH        120     // intervale entre les anims

#define NB_QUART_COEUR      16
#define NB_FLACON           4
#define NB_TROC_GEMME       4
#define NB_GEMME_FORCE      200
#define NB_MEDAILLONS       3
#define NB_CRISTAUX         7
#define NB_POISSONS         30
#define NB_COFFRES          18
#define COFFRES_MAX_SIZE    60
#define NB_COEUR            13
#define NB_DONJON           16
#define NB_MONSTERS         142
#define MAX_ROOMS_X         20
#define MAX_ROOMS_Y         20

#include <sstream>
#include <iostream>
#include <fstream>
#include <cstdlib>

#include "../../config/AudioHelper.h"

using namespace std;

enum Direction {N, S, W, E};

enum Animation{IDLE, WALK, PUSH, TOUCHE, FLOTTE, NAGE, COULE, CHUTE, HIT_EPEE, SPIN_EPEE, NOYE, MORT,
    TROUVE_SIMPLE, TROUVE_DOUBLE, TIR_ARC, PORTE, TRANSPORTE, THROW, SOULEVE, TIR_GRAPPIN,
    TIR_BAGUETTE_FEU, TIR_BAGUETTE_GLACE, TIR_LANTERNE, TIR_MARTEAU, BOIT_POTION, JOUE_OCARINA,
    ELECTRIC, PECHE, BARQUE_IDLE, BARQUE, JUMP_IN_WAGON, WAGON, JUMP_OUT_WAGON};

enum Collision{TROU = -1, HERBE = 0, HERBE_HAUTE, TERRE, GLACE, MURRET, MUR, EAU, EAU_PROF,
    MUR_H, MUR_B, MUR_G, MUR_D, MUR_HG, MUR_HD, MUR_BG, MUR_BD, BOIS, SABLE, NEIGE, PIERRE,
    SOL_BOIS, TAPIS, DANGER, DANGER_BAS, GRAND_MUR};

enum Equipment{ARC, ARC_FEE, PALMES, BOMBES,
    LANTERNE, GRAPPIN, BAGUETTE_FEU,
    BAGUETTE_GLACE, MARTEAU, MASQUE,
    FLACON_1, FLACON_2, FLACON_3, FLACON_4,
    GANTS, GANTS_2, OCARINA, CHANT_1, CHANT_2, CHANT_3,
    CANNE_A_PECHE_1, CANNE_A_PECHE_2, CANNE_A_PECHE_3,
    FLECHE_FEU, FLECHE_GLACE, FLECHE_LUMIERE,
    TROC_GEMME_ROUGE, TROC_GEMME_BLEUE, TROC_GEMME_JAUNE, TROC_GEMME_VERTE,
    RECUP_FLECHE, RECUP_BOMBE, CLE_TEMPS, ARC_OR, DETECTEUR, PERLE, LIVRE,
    BOTTES, ENCYCLOPEDIE, ENCYCLOPEDIE_POISSONS, SAC_TROC,
    BONUS_FORCE_1, BONUS_FORCE_2, BONUS_DEFENSE_1, BONUS_DEFENSE_2,
    CARTE, TRIFORCE, NB_EQPMT};

enum TypeTroc{TT_SAC_RIZ, TT_SAC_OLIVES, TT_BOCAL_EPICES, TT_PERLES, TT_PLUME, TT_POULET, TT_TAPISSERIE,
    TT_AMULETTE, TT_PARFUM, TT_ROC_MAGMA, TT_ROC_GIVRE, TT_POIVRON, TT_ROUAGE, TT_JAUGE, TT_ROUE, TT_SOURCE_NRJ, NB_TROC};

enum TrocGemmeRouge{TGR_GEMME, TGR_ENCLUME, TGR_FER_CHEVAL, TGR_RIEN, TGR_BIJOU, TGR_LETTRE, TGR_ALLIANCE, TGR_RIEN_2, TGR_MARTEAU, TGR_FINI};
enum TrocGemmeBleue{TGB_GEMME, TGB_BOTTES, TGB_MARTEAU, TGB_CLOCHE, TGB_HACHE, TGB_CORDE, TGB_PLAN, TGB_FINI};
enum TrocGemmeJaune{TGJ_GEMME, TGJ_VIN, TGJ_POUDRE_OR, TGJ_MIROIR, TGJ_CUILLERE, TGJ_CRANE, TGJ_REC_DETTE, TGJ_SCIE, TGJ_FINI};
enum TrocGemmeVerte{TGV_GEMME, TGV_CARTE, TGV_BALAIS, TGV_CONFITURE, TGV_CROISSANT, TGV_BIERE, TGV_CLOU, TGV_FINI};

enum TypeAttack{TA_PHYSIC, TA_MAGIC, TA_SWORD, TA_SWORD_HOLD, TA_ARROW, TA_EXPLO, TA_GRAPPIN,
    TA_MARTEAU, TA_MASSIF, TA_NB_MAX};

enum TypeEffect{TE_NORMAL, TE_FEU, TE_GLACE, TE_EAU, TE_GRAVITE, TE_QUASI_MORT, TE_MORT, TE_ELECTRIC, TE_LAPIN, TE_COLERE,
    TE_CONFUSION, TE_ARGENT, TE_EXPONENTIEL, TE_LUMIERE, TE_NB_MAX};

enum TypeItem{TI_PETIT_COEUR, TI_QUART_COEUR, TI_FLECHE, TI_BOMBE, TI_MAGIE_PEU, TI_MAGIE_BEAUCOUP,
    TI_RUBIS_VERT, TI_RUBIS_BLEU, TI_RUBIS_ROUGE,
    TI_GANON_OR_1, TI_GANON_OR_2, TI_GANON_OR_5,
    TI_PIECE_OR_1, TI_PIECE_OR_2, TI_PIECE_OR_3,
    TI_FLACON_1, TI_FLACON_2, TI_FLACON_3, TI_FLACON_4,
    TI_FLECHE_FEU, TI_FLECHE_GLACE, TI_FLECHE_LUMIERE,
    TI_BOUCLIER_1, TI_BOUCLIER_2, TI_BOUCLIER_3, TI_BOUCLIER_4,
    TI_TUNIQUE_BLEUE, TI_TUNIQUE_ROUGE, TI_TUNIQUE_BLANCHE,
    TI_CHANT_1, TI_CHANT_2, TI_CHANT_3, TI_SAC_BOMBES, TI_LIVRE,
    TI_EPEE_1, TI_EPEE_2, TI_EPEE_3, TI_EPEE_4, TI_EPEE_5,
    TI_GANT, TI_GANT2, TI_ARC, TI_ARC_FEE, TI_ARC_OR, TI_COEUR,
    TI_PALMES, TI_OCARINA, TI_MARTEAU, TI_LANTERNE, TI_MASQUE,
    TI_POTION_ROUGE, TI_POTION_VERTE, TI_POTION_BLEUE, TI_CARTE,
    TI_CARTE_DONJON, TI_BOUSSOLE, TI_CLE_BOSS, TI_CLE, TI_DOUBLE_MAGIE,
    TI_GRAPPIN, TI_BAGUETTE_FEU, TI_BAGUETTE_GLACE, TI_BOTTES, TI_PERLE_LUNE,
    TI_INTERRUPTEUR, TI_TRIFORCE, TI_GEMME_FORCE, TI_CRISTAL, TI_PLANCHES,
    TI_ECREVISSE, TI_EPERLAN, TI_POISSON_ZEBRE, TI_GOBIE, TI_CRABE, TI_PIBALE, TI_VAIRON,
    TI_CARASSIN_DORE, TI_ABLETTE, TI_GOUJON, TI_LOCHE, TI_EPINOCHE, TI_GREMILLE, TI_BLENNIE,
    TI_MAGICARPE, TI_TRUITE, TI_AYU, TI_PERCHE_NOIRE, TI_ANGUILLE, TI_POISSON_CHAT, TI_BLACK_BASS,
    TI_HUCHON, TI_BARBEAU, TI_LAMPROIE, TI_HOTU, TI_SAUMON, TI_ESTURGEON, TI_OMBLE_CHEVALIER,
    TI_CARPE, TI_SILURE,
    TI_MEDAILLON_1, TI_MEDAILLON_2, TI_MEDAILLON_3, TI_CLE_TEMPS,
    TI_GEMME_ROUGE, TI_GEMME_BLEUE, TI_GEMME_JAUNE, TI_GEMME_VERTE,
    TI_ENCLUME, TI_FER_CHEVAL, TI_RIEN, TI_BIJOU, TI_LETTRE, TI_ALLIANCE, TI_RIEN_2, TI_MARTEAU_PONT,
    TI_VIEILLE_BOTTE, TI_MARTEAU_LOURD, TI_CLOCHE, TI_HACHE, TI_CORDE, TI_PLAN,
    TI_VIN, TI_POUDRE_OR, TI_MIROIR, TI_CUILLERE, TI_CRANE, TI_REC_DETTE, TI_SCIE,
    TI_CARTE_VOYAGE, TI_BALAIS, TI_CONFITURE, TI_CROISSANT, TI_BIERE, TI_CLOU,
    TI_SAC_RIZ, TI_SAC_OLIVES, TI_BOCAL_EPICES, TI_POULET, TI_PLUME, TI_PERLES, TI_TAPISSERIE,
    TI_AMULETTE, TI_PARFUM, TI_ROC_MAGMA, TI_ROC_GIVRE, TI_POIVRON, TI_ROUAGE, TI_JAUGE, TI_ROUE, TI_SOURCE_NRJ,
    TI_BOURSE_RUBIS_1, TI_BOURSE_RUBIS_2, TI_BOURSE_RUBIS_3, TI_BOURSE_RUBIS_4,
    TI_BOURSE_GANON_OR_1, TI_BOURSE_GANON_OR_2, TI_BOURSE_GANON_OR_3, TI_BOURSE_GANON_OR_4,
    TI_BOURSE_PIECE_OR_1, TI_BOURSE_PIECE_OR_2, TI_BOURSE_PIECE_OR_3, TI_BOURSE_PIECE_OR_4,
    TI_SAC_RIZ_10, TI_BOCAL_EPICES_10, TI_SAC_OLIVES_10,
    TI_DETECTEUR, TI_BONUS_FORCE_1, TI_BONUS_FORCE_2, TI_BONUS_DEFENSE_1, TI_BONUS_DEFENSE_2,
    TI_CANNE_A_PECHE_1, TI_CANNE_A_PECHE_2, TI_CANNE_A_PECHE_3,
    TI_NO_ITEM, TI_NB_MAX};

enum TypeProjectile{TP_ARROW, TP_ARROW_2, TP_ARROW_FIRE, TP_ARROW_ICE, TP_ARROW_LIGHT, TP_ARROW_GOLD, TP_SOUFFLE,
    TP_BAGUETTE_FEU, TP_BAGUETTE_GLACE, TP_FEU_LONG_ENN, TP_BOULE_FEU, TP_ARC_MAGIQUE, TP_BOULET, TP_LAPIN, TP_ECLAIR,
    TP_BIG_FEU, TP_BIG_GLACE, TP_BOULE_DEMI, TP_BOULE_ULTIME, TP_BOULE_COLERE, TP_BOULE_MORT, TP_BOULE_EXPO, TP_IVRE,
    TP_BAT, TP_FINAL, TP_NB_MAX};

enum TypePoisson{TPO_ECREVISSE, TPO_EPERLAN, TPO_POISSON_ZEBRE, TPO_GOBIE, TPO_CRABE, TPO_PIBALE, TPO_VAIRON,
    TPO_CARASSIN_DORE, TPO_ABLETTE, TPO_GOUJON, TPO_LOCHE, TPO_EPINOCHE, TPO_GREMILLE, TPO_BLENNIE,
    TPO_MAGICARPE, TPO_TRUITE, TPO_AYU, TPO_PERCHE_NOIRE, TPO_ANGUILLE, TPO_POISSON_CHAT, TPO_BLACK_BASS,
    TPO_HUCHON, TPO_BARBEAU, TPO_LAMPROIE, TPO_HOTU, TPO_SAUMON, TPO_ESTURGEON, TPO_OMBLE_CHEVALIER};

enum Avancement{AV_START, AV_INTRO_OK, AV_GARDE_VU, AV_PARLE_GARDE_PALAIS, AV_PARLE_GARDE_RESERVE, AV_RESERVE_VIDE, AV_PARLE_ZELDA_PALAIS,
    AV_PARLE_GARDE_CH_FORTE, AV_PARLE_GARDE_SOUS_SOL, AV_ZELDA_SOUS_SOL, AV_RIDEAUX_REPERES, AV_PASSAGE_TROUVE, AV_STELE_LUE,
    AV_PARLE_ZELDA_GROTTE, AV_PARLE_PECHE, AV_BARQUE_LOUEE, AV_PANCARTE_MINE_LUE, AV_PARLE_GARS_MINE_SALOON, AV_PARLE_BARMAN_SALOON,
    AV_APPREND_POUR_ROC_GIVRE, AV_DETTE_PAYEE, AV_GARS_MINE_AIDE, AV_PARLE_GARS_MINE_MINE, AV_ZELDA_POUSSEE, AV_PORTAIL_F_OUVERT,
    AV_PORTAIL_F_UTILISE, AV_GARDES_F_ACHETES, AV_CLE_PRISON_F_TROUVEE, AV_BOWSER_AIDE, AV_MOT_DE_PASSE_CONNU, AV_MOT_DE_PASSE_DONNE,
    AV_MULTIPASS, AV_MULTIPASSE_UTILISE, AV_SAGES_GROTTE, AV_PORTAIL_PRE_PRESENT_OUVERT, AV_PORTAIL_PRE_PRESENT_UTILISE, AV_PLANCHES_TROUVEES,
    AV_PONT_CONSTRUIT, AV_PORTAIL_PRE_PRESENT_FERME, AV_RETOUR_PRESENT, AV_ARRIVE_PASSE, AV_INFO_PASSE_OK, AV_NEIGE_STOPPEE, AV_GANON_CAPTURE,
    AV_GANON_SORTI, AV_GANON_DISPARU, AV_TEMPLE_DEESSES_APPARU, AV_SPEECH_AGAHNIM, AV_TELE_ZELDA, AV_SPEECH_GANON, AV_SPEECH_GANONDORF,
    AV_SPEECH_VINCENT
    };

enum Epoque{EP_PASSE, EP_PRESENT, EP_FUTUR, EP_RIEN
    };



#endif  // CommonGame.h

